class Common:
    PARAM = '--'
    REPORTS = 'reports'


class CI:
    OUTPUT_DIR = 'output_dir'
    OUTPUT_DIR_PARAM = Common.PARAM + OUTPUT_DIR
    VERBOSE = 'verbose'
    VERBOSE_PARAM = Common.PARAM + VERBOSE


class FlaskScript:
    STORE_TRUE = 'store_true'


class Nose:
    COVER_BRANCHES = 'cover-branches'
    COVER_BRANCHES_PARAM = Common.PARAM + COVER_BRANCHES
    COVER_HTML = 'cover-html'
    COVER_HTML_PARAM = Common.PARAM + COVER_HTML
    COVER_HTML_DIR = 'cover-html'
    COVER_HTML_DIR_PARAM = Common.PARAM + COVER_HTML_DIR
    COVER_PACKAGE = 'cover-package'
    COVER_PACKAGE_PARAM = Common.PARAM + COVER_PACKAGE
    COVER_XML = 'cover-xml'
    COVER_XML_PARAM = Common.PARAM + COVER_XML
    COVER_XML_FILE = 'cover-xml-file'
    COVER_XML_FILE_PARAM = Common.PARAM + COVER_XML_FILE
    COVERAGE_FILE = '.coverage'
    NOSE_TESTS = 'nosetests'
    WITH_COVERAGE = 'with-coverage'
    WITH_COVERAGE_PARAM = Common.PARAM + WITH_COVERAGE
    WITH_XUNIT = 'with-xunit'
    WITH_XUNIT_PARAM = Common.PARAM + WITH_XUNIT
    XUNIT_FILE = 'xunit-file'
    XUNIT_FILE_PARAM = Common.PARAM + XUNIT_FILE


class Pep8:
    CONFIG_FILE = 'config_file'
    PEP8_MAX_LINE_LENGTH = 'pep8-max-line-length'
    PEP8_MAX_LINE_LENGTH_PARAM = Common.PARAM + PEP8_MAX_LINE_LENGTH
    PEP8_RC_FILE = 'pep8-rcfile'
    PEP8_RC_FILE_PARAM = Common.PARAM + PEP8_RC_FILE
    MAX_LINE_LENGTH = 'max_line_length'


class PyLint:
    PYLINT_RC = 'pylint.rc'
    PYLINT_RC_FILE = 'pylint-rcfile'
    PYLINT_RC_FILE_PARAM = Common.PARAM + 'pylint-rcfile'
    RC_FILE = 'rcfile'
    RC_FILE_PARAM = Common.PARAM + RC_FILE


class Reports:
    COVERAGE = Common.REPORTS + '/coverage.xml'
    HTML_COVERAGE = Common.REPORTS + '/html-coverage'
    NOSE_TESTS = Common.REPORTS + '/nosetests.xml'
    PEP8_REPORT = 'pep8.report'
    PYLINT_REPORT = 'pylint.report'


class Settings:
    CI_TASKS = 'CI_TASKS'
    PEP8_RC_FILE = 'PEP8_RCFILE'
    PROJECT_APPS = 'PROJECT_APPS'
    PYLINT_RC_FILE = 'PYLINT_RCFILE'



# -*- coding: utf-8 -*-
"""
    flask_ci
    ~~~~~~~~~~~

    Continuous Integration support for Flask

    :copyright: (c) 2016 by Vicente Neto.
    :license: MIT, see LICENSE for more details.
"""
import os
from importlib import import_module

from flask_script import Command, Option

from flask_ci.util.constants import CI, Common, FlaskScript, Settings


class CICommand(Command):
    """
    Perform CI operations
    :param settings: settings
    """

    help = description = 'Perform CI operations.'

    def __init__(self, settings):
        self.tasks_cls = [import_module(module_name).Reporter for module_name in self.get_task_list(settings)]
        self.tasks = [task_cls() for task_cls in self.tasks_cls]
        self.settings = settings

    @staticmethod
    def get_task_list(settings):
        return getattr(settings, Settings.CI_TASKS, ())

    def get_options(self):
        options = [
            Option('-o', CI.OUTPUT_DIR_PARAM, dest=CI.OUTPUT_DIR, default=Common.REPORTS),
            Option('-v', CI.VERBOSE_PARAM, action=FlaskScript.STORE_TRUE, dest=CI.VERBOSE, default=False)
        ]

        for task in self.tasks:
            if hasattr(task, 'get_arguments'):
                options = options + task.get_arguments()

        return options

    def __call__(self, *args, **kwargs):
        output_dir = kwargs[CI.OUTPUT_DIR]
        verbose = kwargs[CI.VERBOSE]

        if not os.path.exists(output_dir):
            if verbose:
                print('Creating output directory...')
            os.makedirs(output_dir)

        for task in self.tasks:
            if verbose:
                print('Executing {0}...'.format(task.__module__))
            task.run(self.settings, **kwargs)

        if verbose:
            print('Done')



import os

from flask_script import Option

from flask_ci.util.constants import FlaskScript, Nose, Reports, Settings


class Reporter(object):
    @staticmethod
    def get_arguments():
        return [
            Option(Nose.WITH_XUNIT_PARAM, action=FlaskScript.STORE_TRUE, dest=Nose.WITH_XUNIT, default=False),
            Option(Nose.XUNIT_FILE_PARAM, dest=Nose.XUNIT_FILE, default=Reports.NOSE_TESTS),
            Option(Nose.COVER_XML_PARAM, action=FlaskScript.STORE_TRUE, dest=Nose.COVER_XML, default=False),
            Option(Nose.COVER_XML_FILE_PARAM, dest=Nose.COVER_XML_FILE, default=Reports.COVERAGE),
            Option(Nose.COVER_HTML_PARAM, action=FlaskScript.STORE_TRUE, dest=Nose.COVER_HTML, default=False),
            Option(Nose.COVER_HTML_DIR_PARAM, dest=Nose.COVER_HTML_DIR, default=Reports.HTML_COVERAGE),
            Option(Nose.COVER_BRANCHES_PARAM, action=FlaskScript.STORE_TRUE, dest=Nose.COVER_BRANCHES, default=False)
        ]

    def run(self, settings, **options):
        args = list()

        if options[Nose.WITH_XUNIT]:
            args.append(Nose.WITH_XUNIT_PARAM)
            args.append('%s=%s' % (Nose.XUNIT_FILE_PARAM, options[Nose.XUNIT_FILE]))
        if options[Nose.COVER_XML] or options[Nose.COVER_HTML]:
            args.append(Nose.WITH_COVERAGE_PARAM)
            args.append('%s=%s' % (Nose.COVER_PACKAGE_PARAM, ','.join(getattr(settings, Settings.PROJECT_APPS, []))))
        if options[Nose.COVER_XML]:
            args.append(Nose.COVER_XML_PARAM)
            args.append('%s=%s' % (Nose.COVER_XML_FILE_PARAM, options[Nose.COVER_XML_FILE]))
        if options[Nose.COVER_HTML]:
            args.append(Nose.COVER_HTML_PARAM)
            args.append('%s=%s' % (Nose.COVER_HTML_DIR_PARAM, options[Nose.COVER_HTML_DIR]))
        if options[Nose.COVER_BRANCHES]:
            args.append(Nose.COVER_BRANCHES_PARAM)

        command = '%s %s' % (Nose.NOSE_TESTS, ' '.join(args))
        os.system(command)

        if Nose.WITH_COVERAGE_PARAM in args:
            os.remove(Nose.COVERAGE_FILE)



import os.path

import pep8
from flask_script import Option

from flask_ci.util.constants import CI, Pep8, Reports, Settings


class Reporter(object):
    @staticmethod
    def get_arguments():
        return [
            Option(Pep8.PEP8_RC_FILE_PARAM, dest=Pep8.PEP8_RC_FILE),
            Option(Pep8.PEP8_MAX_LINE_LENGTH_PARAM, dest=Pep8.PEP8_MAX_LINE_LENGTH, type=int, default=120)
        ]

    @staticmethod
    def get_config_path(settings, options):
        if options.get(Pep8.PEP8_RC_FILE, None):
            return options[Pep8.PEP8_RC_FILE]

        rcfile = getattr(settings, Settings.PEP8_RC_FILE, '')
        if os.path.exists(rcfile):
            return rcfile

    def run(self, settings, **options):
        output = open(os.path.join(options[CI.OUTPUT_DIR], Reports.PEP8_REPORT), 'w')

        class JenkinsReport(pep8.BaseReport):
            def error(self, line_number, offset, text, check):
                code = super(JenkinsReport, self).error(line_number, offset, text, check)
                if code:
                    source_line = self.line_offset + line_number
                    output.write('%s:%s:%s: %s\n' % (self.filename, source_line, offset + 1, text))

        pep8_options = {}
        config_file = self.get_config_path(settings, options)
        if config_file is not None:
            pep8_options[Pep8.CONFIG_FILE] = config_file

        pep8_options[Pep8.MAX_LINE_LENGTH] = options[Pep8.PEP8_MAX_LINE_LENGTH]

        pep8style = pep8.StyleGuide(
            parse_argv=False,
            reporter=JenkinsReport,
            **pep8_options)

        pep8style.options.report.start()
        for location in getattr(settings, Settings.PROJECT_APPS, []):
            pep8style.input_dir(os.path.relpath(location))
        pep8style.options.report.stop()

        output.close()



import os

from flask_script import Option
from pylint import lint
from pylint.reporters.text import TextReporter

from flask_ci.util.constants import CI, PyLint, Reports, Settings


class ParseableTextReporter(TextReporter):
    name = 'parseable'
    line_format = '{path}:{line}: [{msg_id}({symbol}), {obj}] {msg}'


class Reporter(object):
    @staticmethod
    def get_arguments():
        return [
            Option(PyLint.PYLINT_RC_FILE, dest=PyLint.PYLINT_RC_FILE)
        ]

    @staticmethod
    def get_config_path(settings, options):
        if options.get('pylint_rcfile', None):
            return options[PyLint.PYLINT_RC_FILE]

        rcfile = getattr(settings, Settings.PYLINT_RC_FILE, PyLint.PYLINT_RC)
        if os.path.exists(rcfile):
            return rcfile

        # use built-in
        root_dir = os.path.normpath(os.path.dirname(__file__))
        return os.path.join(root_dir, PyLint.PYLINT_RC)

    def run(self, settings, **options):
        output = open(os.path.join(options[CI.OUTPUT_DIR], Reports.PYLINT_REPORT), 'w')

        args = list()
        args.append('%s=%s' % (PyLint.RC_FILE_PARAM, self.get_config_path(settings, options)))
        args += getattr(settings, Settings.PROJECT_APPS, [])

        lint.Run(args, reporter=ParseableTextReporter(output=output), exit=False)

        output.close()



"""
Flask-CI
--------

Provide Continuous Integration support operations for Flask apps.

Links
`````

* `documentation <https://pythonhosted.org/Flask-CI>`_
* `development version <http://github.com/vicenteneto/flask-ci/zipball/master#egg=Flask-CI-dev>`_
"""
from setuptools import setup

version = '1.1.24'
packages = ['flask_ci', 'flask_ci.tasks', 'flask_ci.util']
install_requires = ['Coverage', 'Flask-Script', 'Nose', 'Pep8', 'Pylint', 'Pylint-Flask']
flask_ci_pkg_data = ['tasks/pylint.rc']

setup(
    name='Flask-CI',
    version=version,
    description='Continuous Integration support for Flask',
    url='https://github.com/vicenteneto/flask-ci',
    author='Vicente Neto',
    author_email='sneto.vicente@gmail.com',
    license='MIT',
    packages=packages,
    install_requires=install_requires,
    package_data={'flask_ci': flask_ci_pkg_data},
    zip_safe=False,
    keywords=['ci', 'jenkins', 'hudson', 'flask', 'pylint', 'pep8', 'coverage', 'nose'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Flask',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Software Development :: Testing'
    ]
)
